package de.luh.dcsec.usecpassboard;

import java.util.List;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ArrayAdapter;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TabHost;
import android.widget.TabHost.TabSpec;
import android.widget.TextView;
import de.luh.dcsec.usecpassboard.LoginDialog.ILoginDialogListener;
import de.luh.dcsec.usecpassboard.SetPasswordDialog.ISetPasswordDialogListener;

public class CredentialPicker extends ListActivity {
	public static final String TAG = "CredentialPicker";
	private Cursor mCursor;
	private String currentActivity;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_credential_picker);
		// Show the Up button in the action bar.
		// getActionBar().setDisplayHomeAsUpEnabled(true);

		Intent intent = getIntent();
		currentActivity = intent.getStringExtra(USecPassBoard.EXTRA_ACTIVITY);

		if (currentActivity != null && currentActivity.length() > 0) {

			/*
			 * CredentialsDataSource db = new CredentialsDataSource(
			 * getApplicationContext());
			 * 
			 * db.open();
			 */

			CredentialsDataSource db = CredentialsDataSource
					.getInstance(getApplicationContext());
			// db.close();

			if (db.isOpen()) {
				setupList();
			} else {
				showPasswordDialog(null);
			}
		}
	}

	private void showPasswordDialog(Bundle data) {
		LoginDialog loginDialog = new LoginDialog(this, new ILoginDialogListener() {
			
			@Override
			public void onSuccess() {
				setupList();
			}
			
			@Override
			public void onCancel() {
				CredentialPicker.this.finish();
			}
		});

		loginDialog.show();
	}

	private void setupList() {
		CredentialsDataSource db = CredentialsDataSource.getInstance(getApplicationContext());
		mCursor = db.getCredentialsByContext(currentActivity);

		startManagingCursor(mCursor);

		ListAdapter adapter = new SimpleCursorAdapter(this,
				R.layout.credentials_list, mCursor, new String[] {
						MySQLiteHelper.COLUMN_USERNAME,
						MySQLiteHelper.COLUMN_ACTIVITY }, new int[] {
						R.id.username, R.id.context });

		setListAdapter(adapter);

		registerForContextMenu(getListView());
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		Log.d(TAG, "item clicked");

		Cursor c = ((SimpleCursorAdapter) l.getAdapter()).getCursor();
		c.moveToPosition(position);

		Intent intent = new Intent(DialogActivity.MESSAGE);
		intent.putExtra(USecPassBoard.EXTRA_ACTIVITY, c.getString(1));
		intent.putExtra(USecPassBoard.EXTRA_USERNAME, c.getString(2));
		intent.putExtra(USecPassBoard.EXTRA_PASSWORD, c.getString(3));

		sendBroadcast(intent);

		super.onListItemClick(l, v, position, id);

		finish();
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
		Log.d(TAG, "context menu created");

		getMenuInflater().inflate(R.menu.activity_credential_detail, menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_credential_delete:
			AdapterContextMenuInfo menuInfo = (AdapterContextMenuInfo) item
					.getMenuInfo();

			Log.d(TAG, "selected(menu): " + menuInfo.id);
			Log.d(TAG, "delete credential");

			long credentialId = menuInfo.id;

			confirmDeleteCredential(credentialId);

			break;
		case R.id.menu_credential_add_context:
			Log.d(TAG, "add credential context");
			showAddContextDialog();
			break;
		case R.id.menu_database_close:
			Log.i(TAG, "close db");
			CredentialsDataSource.getInstance(getApplicationContext()).close();
			finish();
			break;
		case R.id.menu_database_rekey:
			showRekeyDialog(null);
			break;
		case R.id.menu_credential_delete_all:
			confirmDeleteAllCredentials();
			break;
		}

		return super.onContextItemSelected(item);
	}

	private void showAddContextDialog() {
		LayoutInflater inflater = LayoutInflater.from(this);
		View v = inflater.inflate(R.layout.dialog_add_context, null);
		TabHost th = (TabHost) v.findViewById(android.R.id.tabhost);
		th.setup();
		TabSpec t1 = th.newTabSpec("Webs");
		t1.setIndicator("Web");
		t1.setContent(R.id.tab1);
		th.addTab(t1);
		TabSpec t2 = th.newTabSpec("Apps");
		t2.setIndicator("App");
		t2.setContent(R.id.tab2);
		th.addTab(t2);
		th.setCurrentTab(0);
		
		ListView lv = (ListView) v.findViewById(R.id.listView1);
		
		
		
		final PackageManager pm = getPackageManager();
		
		final List<ApplicationInfo> installedApps = pm.getInstalledApplications(PackageManager.GET_META_DATA);
		
		ListAdapter adapter = new ArrayAdapter<ApplicationInfo>(
				getApplicationContext(), R.layout.credentials_list, installedApps) {
			ViewHolder holder;

			class ViewHolder {
				TextView username;
				TextView context;
			}

			@Override
			public View getView(int position, View convertView,
					ViewGroup parent) {
				final LayoutInflater inflater = (LayoutInflater) getApplicationContext()
						.getSystemService(
								Context.LAYOUT_INFLATER_SERVICE);

				if (convertView == null) {
					convertView = inflater.inflate(
							R.layout.credentials_list, null);

					holder = new ViewHolder();
					holder.username = (TextView) convertView
							.findViewById(R.id.username);
					holder.context = (TextView) convertView
							.findViewById(R.id.context);
					convertView.setTag(holder);
				} else {
					// view already defined, retrieve view holder
					holder = (ViewHolder) convertView.getTag();
				}

				holder.username
						.setText(installedApps.get(position).loadLabel(pm));
				holder.context.setText(installedApps.get(position).packageName);

				return convertView;
			}
		};
		
		lv.setAdapter(adapter);
		
		AlertDialog.Builder b = new AlertDialog.Builder(this);
		b/*.setTitle("Add").setMessage("add con")*/.setView(v).setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				Log.d(TAG, "add con");
			}
		});
		b.show();
	}

	private void confirmDeleteAllCredentials() {
		AlertDialog.Builder b = new AlertDialog.Builder(this);
		b.setTitle(R.string.title_dialog_credentials_delete)
				.setMessage(R.string.message_dialog_credentials_delete)
				.setIcon(android.R.drawable.ic_dialog_alert)
				.setPositiveButton(android.R.string.yes,
						new DialogInterface.OnClickListener() {

							@Override
							public void onClick(DialogInterface dialog,
									int which) {
								CredentialsDataSource db = CredentialsDataSource
										.getInstance(getApplicationContext());
								db.deleteAllCredential();
								refresh(db);
							}
						})
				.setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {

					@Override
					public void onClick(DialogInterface dialog, int which) {

					}
				});
		b.show();
	}

	private void showRekeyDialog(Bundle data) {
		SetPasswordDialog setPasswordDialog = new SetPasswordDialog(this, new ISetPasswordDialogListener() {
			
			@Override
			public void onSuccess() {
				Log.d(TAG, "rekeying successfully finished");
			}
			
			@Override
			public void onCancel() {
				Log.d(TAG, "rekeying canceled");
			}
		});
		
		setPasswordDialog.show();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_credential_picker, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// This ID represents the Home or Up button. In the case of this
			// activity, the Up button is shown. Use NavUtils to allow users
			// to navigate up one level in the application structure. For
			// more details, see the Navigation pattern on Android Design:
			//
			// http://developer.android.com/design/patterns/navigation.html#up-vs-back
			//
			// NavUtils.navigateUpFromSameTask(this);
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	public void closeDialog(View v) {
		this.finish();
	}

	public void confirmDeleteCredential(final long credentialId) {
		final AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.title_dialog_credential_delete)
				.setMessage(R.string.message_dialog_credential_delete)
				.setIcon(android.R.drawable.ic_dialog_alert)
				.setPositiveButton(android.R.string.yes,
						new DialogInterface.OnClickListener() {

							@Override
							public void onClick(DialogInterface dialog,
									int which) {
								CredentialsDataSource db = CredentialsDataSource
										.getInstance(getApplicationContext());

								db.deleteCredential(credentialId);

								refresh(db);
							}
						}).setNegativeButton(android.R.string.no, null);

		builder.show();
	}

	/**
	 * @param db
	 */
	private void refresh(CredentialsDataSource db) {
		((SimpleCursorAdapter) getListAdapter()).changeCursor(db
				.getCredentialsByContext(currentActivity));
	}

}
