package de.luh.dcsec.usecpassboard;

import java.util.ArrayList;
import java.util.List;

import net.sqlcipher.database.SQLiteDatabase;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.util.Log;

public class CredentialsDataSource {

	// Database fields
	private SQLiteDatabase database;
	private MySQLiteHelper dbHelper;
	private static CredentialsDataSource instance;
	private Context ctx;
	private String[] allColumns = { MySQLiteHelper.COLUMN_ID,
			MySQLiteHelper.COLUMN_ACTIVITY, MySQLiteHelper.COLUMN_USERNAME,
			MySQLiteHelper.COLUMN_PASSWORD };

	public CredentialsDataSource(Context context) {
		this.ctx = context;
	}

	public static synchronized CredentialsDataSource getInstance(Context ctx) {

		if (instance == null) {
			instance = new CredentialsDataSource(ctx);
		}

		return instance;
	}

	public CredentialsDataSource open(String password) throws SQLException {
		dbHelper = new MySQLiteHelper(this.ctx);
		database = dbHelper.getWritableDatabase(password);
		System.gc();
		return this;
	}

	public boolean isOpen() {
		if (database != null)
			return database.isOpen();
		else
			return false;
	}
    
    public void rekey (String password)
    {
    	database.execSQL("PRAGMA rekey = '" + password + "'");
    	System.gc();
    }

	public void close() {
		if (dbHelper != null) {
			dbHelper.close();
			instance = null;
			database.close();
		}
	}

	public Credential createCredential(String activity, String username,
			String password) {
		ContentValues values = new ContentValues();
		values.put(MySQLiteHelper.COLUMN_ACTIVITY, activity);
		values.put(MySQLiteHelper.COLUMN_USERNAME, username);
		values.put(MySQLiteHelper.COLUMN_PASSWORD, password);
		long insertId = database.insert(MySQLiteHelper.TABLE_COMMENTS, null,
				values);
		Cursor cursor = database.query(MySQLiteHelper.TABLE_COMMENTS,
				allColumns, MySQLiteHelper.COLUMN_ID + " = " + insertId, null,
				null, null, null);
		cursor.moveToFirst();
		Credential newCredential = cursorToCredential(cursor);
		cursor.close();
		return newCredential;
	}

	public void deleteCredential(Credential credential) {
		long id = credential.getId();
		System.out.println("Credential deleted with id: " + id);
		database.delete(MySQLiteHelper.TABLE_COMMENTS, MySQLiteHelper.COLUMN_ID
				+ " = " + id, null);
	}

	public void deleteCredential(long credentialId) {
		System.out.println("Credential deleted with id: " + credentialId);
		database.delete(MySQLiteHelper.TABLE_COMMENTS, MySQLiteHelper.COLUMN_ID
				+ " = " + credentialId, null);
	}

	public void deleteAllCredential() {
		System.out.println("All Credentials deleted with id: ");
		database.delete(MySQLiteHelper.TABLE_COMMENTS, null, null);
	}

	public List<Credential> getAllCredentials() {
		List<Credential> credentials = new ArrayList<Credential>();

		Cursor cursor = database.query(MySQLiteHelper.TABLE_COMMENTS,
				allColumns, null, null, null, null, null);

		cursor.moveToFirst();
		while (!cursor.isAfterLast()) {
			Credential credential = cursorToCredential(cursor);
			credentials.add(credential);
			cursor.moveToNext();
		}
		// Make sure to close the cursor
		cursor.close();
		return credentials;
	}

	public List<Credential> getCredentials(String activity) {
		List<Credential> credentials = new ArrayList<Credential>();

		Cursor cursor = database.query(MySQLiteHelper.TABLE_COMMENTS,
				allColumns, MySQLiteHelper.COLUMN_ACTIVITY + " = " + "?",
				new String[] { activity }, null, null, null);

		cursor.moveToFirst();
		while (!cursor.isAfterLast()) {
			Credential credential = cursorToCredential(cursor);
			credentials.add(credential);
			cursor.moveToNext();
		}
		// Make sure to close the cursor
		cursor.close();
		return credentials;
	}

	public Cursor getCredentialsByContext(String activity) {

		Cursor cursor = database.query(MySQLiteHelper.TABLE_COMMENTS,
				allColumns, MySQLiteHelper.COLUMN_ACTIVITY + " = " + "?",
				new String[] { activity }, null, null, null);

		return cursor;
	}

	public List<Credential> getCredentials(String activity, String username) {
		List<Credential> credentials = new ArrayList<Credential>();

		Cursor cursor = database.query(MySQLiteHelper.TABLE_COMMENTS,
				allColumns, MySQLiteHelper.COLUMN_ACTIVITY + " = " + "?"
						+ " and " + MySQLiteHelper.COLUMN_USERNAME + " = "
						+ "?", new String[] { activity, username }, null, null,
				null);

		cursor.moveToFirst();
		while (!cursor.isAfterLast()) {
			Credential credential = cursorToCredential(cursor);
			credentials.add(credential);
			cursor.moveToNext();
		}
		// Make sure to close the cursor
		cursor.close();
		return credentials;
	}

	public Credential getCredentialById(String id) {
		
		Log.d("CredentialDb", "select id: " + id);

		Cursor cursor = database.query(MySQLiteHelper.TABLE_COMMENTS,
				allColumns, MySQLiteHelper.COLUMN_ID + " = " + "?", new String[] { id }, null, null,
				null);

		cursor.moveToFirst();
		while (!cursor.isAfterLast()) {
			Credential credential = cursorToCredential(cursor);
			return credential;
		}
		return null;
	}

	private Credential cursorToCredential(Cursor cursor) {
		Credential credential = new Credential();
		credential.setId(cursor.getLong(0));
		credential.setActivity(cursor.getString(1));
		credential.setUsername(cursor.getString(2));
		credential.setPassword(cursor.getString(3));
		return credential;
	}
}