package de.luh.dcsec.usecpassboard;

import net.sqlcipher.SQLException;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

public class LoginDialog extends AlertDialog implements
		DialogInterface.OnShowListener, DialogInterface.OnCancelListener {
	private static final String TAG = "LoginDialog";
	private Context mContext;
	private View mLoginView;
	private EditText mPasswordField;
	private boolean mAlertReady = false;
	private ILoginDialogListener mListener;

	protected LoginDialog(Context context, ILoginDialogListener listener) {
		super(context , android.R.style.Theme_Holo_Dialog);

		this.mContext = context;
		this.setOnShowListener(this);
		this.setOnCancelListener(this);
		this.mListener = listener;
	}

	protected void onCreate(Bundle savedInstanceState) {
		Log.d(TAG, "onCreate");

		LayoutInflater inflater = getLayoutInflater();
		this.mLoginView = inflater.inflate(R.layout.dialog_login, null);
		this.mPasswordField = (EditText) mLoginView
				.findViewById(R.id.loginPassword);

		setTitle(R.string.title_dialog_login);
		setMessage(mContext.getText(R.string.message_dialog_login));
		setIcon(android.R.drawable.ic_lock_idle_lock);
		setView(mLoginView);

		setButton(BUTTON_NEGATIVE, mContext.getText(android.R.string.cancel),
				new DialogInterface.OnClickListener() {

					@Override
					public void onClick(DialogInterface dialog, int which) {
						// CredentialPicker.this.finish();
						mListener.onCancel();
					}
				});
		setButton(BUTTON_POSITIVE, mContext.getText(R.string.button_dialog_login),
				new DialogInterface.OnClickListener() {

					@Override
					public void onClick(DialogInterface dialog, int which) {
						// TODO Auto-generated method stub

					}
				});
		super.onCreate(savedInstanceState);
	}

	@Override
	public void onShow(DialogInterface dialog) {
		Log.d(TAG, "onShow");
		if (mAlertReady == false) {
			// set View.OnClickListener to prevent dialog from closing
			Button button = getButton(BUTTON_POSITIVE);
			button.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					String password = mPasswordField.getText().toString();

					try {
						CredentialsDataSource
								.getInstance(mContext.getApplicationContext())
								.open(password);
						mListener.onSuccess();
						dismiss();
					} catch (SQLException e) {
						mPasswordField.setError(mContext.getText(R.string.error_dialog_login_password));
					}
				}
			});
			mAlertReady = true;
		}
	}

	@Override
	public void onCancel(DialogInterface dialog) {
		Log.d(TAG, "onCancel");
		this.mListener.onCancel();

	}

	public interface ILoginDialogListener {
		public void onCancel();

		public void onSuccess();
	}

}
