package de.luh.dcsec.usecpassboard;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;
import de.luh.dcsec.usecpassboard.R;

public class SetPasswordDialog extends AlertDialog implements
		DialogInterface.OnShowListener, DialogInterface.OnCancelListener {

	private static final String TAG = "SetPasswordDialog";
	private Context mContext;
	private View mSetPasswordView;
	private EditText mPasswordField;
	private EditText mPasswordRepeatField;
	private boolean mAlertReady = false;
	private ISetPasswordDialogListener mListener;

	protected SetPasswordDialog(Context context,
			ISetPasswordDialogListener listener) {
		super(context, android.R.style.Theme_Holo_Dialog);

		this.mContext = context;
		this.setOnShowListener(this);
		this.setOnCancelListener(this);
		this.mListener = listener;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Log.d(TAG, "onCreate");

		LayoutInflater inflater = getLayoutInflater();
		mSetPasswordView = inflater.inflate(R.layout.dialog_setpassword, null);
		mPasswordField = (EditText) mSetPasswordView
				.findViewById(R.id.newPassword);
		mPasswordRepeatField = (EditText) mSetPasswordView
				.findViewById(R.id.newPasswordRepeated);

		setTitle(R.string.title_dialog_set_password);
		setMessage(mContext.getText(R.string.message_dialog_set_password));
		setIcon(android.R.drawable.ic_lock_lock);
		setView(mSetPasswordView);
		setButton(BUTTON_NEGATIVE, mContext.getText(android.R.string.cancel),
				new DialogInterface.OnClickListener() {

					@Override
					public void onClick(DialogInterface dialog, int which) {
						mListener.onCancel();
					}
				});
		setButton(BUTTON_POSITIVE, mContext.getText(android.R.string.ok), new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub

			}
		});

		/*
		 * .setPositiveButton("Change", new DialogInterface.OnClickListener() {
		 * 
		 * @Override public void onClick(DialogInterface dialog, int which) {
		 * EditText passwordField = (EditText) rekeyView
		 * .findViewById(R.id.newPassword); EditText passwordRepeatField =
		 * (EditText) rekeyView .findViewById(R.id.newPasswordRepeated);
		 * 
		 * String newPassword = passwordField.getText() .toString(); String
		 * newPasswordRepeated = passwordRepeatField .getText().toString();
		 * 
		 * Bundle d = new Bundle();
		 * 
		 * if (newPassword == null) { d.putString("" + R.id.newPassword,
		 * "Password is required"); } else if (newPassword.length() < 2) {
		 * d.putString("" + R.id.newPassword, "Password is too short"); }
		 * 
		 * if (newPassword != null && !newPassword .equals(newPasswordRepeated))
		 * { d.putString("" + R.id.newPasswordRepeated,
		 * "Passwords do not match"); }
		 * 
		 * if (newPassword != null && newPassword.length() > 0 && newPassword
		 * .equals(newPasswordRepeated)) { CredentialsDataSource db =
		 * CredentialsDataSource .getInstance(getApplicationContext());
		 * db.rekey(newPassword);
		 * 
		 * AlertDialog.Builder ab = new AlertDialog.Builder(
		 * CredentialPicker.this); ab.setTitle("Password changed successfully")
		 * .setMessage( "Your password has been changed successfully.")
		 * .setPositiveButton( "Ok", new DialogInterface.OnClickListener() {
		 * 
		 * @Override public void onClick( DialogInterface dialog, int which) {
		 * // TODO // Auto-generated // method stub
		 * 
		 * } }); ab.show(); } else { showRekeyDialog(d); } } })
		 */

		super.onCreate(savedInstanceState);
	}

	@Override
	public void onCancel(DialogInterface dialog) {
		Log.d(TAG, "onCancel");
		mListener.onCancel();
	}

	@Override
	public void onShow(DialogInterface dialog) {
		Log.d(TAG, "onShow");
		if (mAlertReady == false) {
			// set View.OnClickListener to prevent dialog from closing
			// automatically
			Button button = getButton(BUTTON_POSITIVE);
			button.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					String newPassword = mPasswordField.getText().toString();
					String newPasswordRepeated = mPasswordRepeatField.getText()
							.toString();

					boolean error = false;

					if (newPassword == null) {
						mPasswordField.setError(mContext.getText(R.string.error_dialog_set_password_password_required));
						error = true;
					} else if (newPassword.length() < 2) {
						mPasswordField.setError(mContext.getText(R.string.error_dialog_set_password_password_too_short));
						error = true;
					}
					if (newPassword != null
							&& !newPassword.equals(newPasswordRepeated)) {
						mPasswordRepeatField.setError(mContext.getText(R.string.error_dialog_set_password_password_not_match));
						error = true;
					}

					if (!error) {
						CredentialsDataSource db = CredentialsDataSource
								.getInstance(mContext.getApplicationContext());

						SharedPreferences prefs = PreferenceManager
								.getDefaultSharedPreferences(mContext
										.getApplicationContext());

						boolean firstTime = prefs
								.getBoolean("first_time", true);

						if (firstTime) {
							db.open(newPassword);
							db.close();

							Editor editor = prefs.edit();
							editor.putBoolean("first_time", false);
							editor.commit();
						} else {
							db.rekey(newPassword);
						}

						/*AlertDialog.Builder ab = new AlertDialog.Builder(
								mContext);
						ab.setTitle("Password set successfully")
								.setMessage(
										"Your password has been set successfully.")
								.setPositiveButton("Ok",
										new DialogInterface.OnClickListener() {

											@Override
											public void onClick(
													DialogInterface dialog,
													int which) {
												mListener.onSuccess();
												SetPasswordDialog.this
														.dismiss();
											}
										});
						ab.show();
						*/
						Toast.makeText(mContext, R.string.message_dialog_set_password_success, Toast.LENGTH_LONG).show();
						mListener.onSuccess();
						SetPasswordDialog.this.dismiss();

					} else {
					}
				}
			});
			mAlertReady = true;
		}

	}

	public interface ISetPasswordDialogListener {
		public void onCancel();

		public void onSuccess();
	}

}
